# Generated by Django 5.1.1 on 2024-10-07 14:58

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("django_logbox", "0002_remove_serverlog_duration"),
    ]

    operations = [
        migrations.RenameField(
            model_name="serverlog",
            old_name="exception",
            new_name="exception_type",
        ),
        migrations.AddField(
            model_name="serverlog",
            name="exception_message",
            field=models.TextField(null=True, verbose_name="exception_message"),
        ),
    ]
